@@ -256,6 +256,7 @@ void i_stream_snapshot_free(struct istream_snapshot **_snapshot)
 	else {
 		if (snapshot->old_memarea != NULL)
 			memarea_unref(&snapshot->old_memarea);
+		i_stream_unref(&snapshot->istream);
 		i_free(snapshot);
 	}
 }
@@ -794,6 +795,27 @@ int i_stream_read_data(struct istream *stream, const unsigned char **data_r,
 	return -1;
 }
 
+int i_stream_read_limited(struct istream *stream, const unsigned char **data_r,
+			  size_t *size_r, size_t limit)
+{
+	struct istream_private *_stream = stream->real_stream;
+	int ret;
+
+	*data_r = i_stream_get_data(stream, size_r);
+	if (*size_r >= limit) {
+		*size_r = limit;
+		return 1;
+	}
+
+	_stream->data_limit = limit;
+	ret = i_stream_read_more(stream, data_r, size_r);
+	_stream->data_limit = 0;
+
+	if (*size_r >= limit)
+		*size_r = limit;
+	return ret;
+}
+
 void i_stream_compress(struct istream_private *stream)
 {
 	i_assert(stream->memarea == NULL ||
@@ -869,6 +891,7 @@ bool i_stream_try_alloc(struct istream_private *stream,
 			size_t wanted_size, size_t *size_r)
 {
 	i_assert(wanted_size > 0);
+	i_assert(stream->buffer_size >= stream->pos);
 
 	if (wanted_size > stream->buffer_size - stream->pos) {
 		if (stream->skip > 0) {
@@ -887,10 +910,18 @@ bool i_stream_try_alloc(struct istream_private *stream,
 		}
 	}
 
-	*size_r = stream->buffer_size - stream->pos;
-	if (stream->try_alloc_limit > 0 &&
-	    *size_r > stream->try_alloc_limit)
-		*size_r = stream->try_alloc_limit;
+	if (stream->data_limit == 0 ||
+	    (stream->buffer_size - stream->skip) < stream->data_limit)
+		*size_r = stream->buffer_size - stream->pos;
+	else {
+		size_t buffered = (stream->pos - stream->skip);
+
+		if (buffered >= stream->data_limit)
+			*size_r = 0;
+		else
+			*size_r = stream->data_limit - buffered;
+	}
+	i_assert(stream->w_buffer != NULL || *size_r == 0);
 	return *size_r > 0;
 }
 
@@ -926,6 +957,18 @@ void *i_stream_alloc(struct istream_private *stream, size_t size)
 	return stream->w_buffer + stream->pos;
 }
 
+void i_stream_memarea_detach(struct istream_private *stream)
+{
+	if (stream->memarea != NULL) {
+		/* Don't overwrite data in a snapshot. Allocate a new
+		   buffer instead. */
+		memarea_unref(&stream->memarea);
+		stream->buffer_size = 0;
+		stream->buffer = NULL;
+		stream->w_buffer = NULL;
+	}
+}
+
 bool i_stream_add_data(struct istream *_stream, const unsigned char *data,
 		       size_t size)
 {
