@@ -16,6 +16,7 @@
 #include "fts-plugin.h"
 #include "fts-user.h"
 #include "fts-storage.h"
+#include "hash.h"
 
 
 #define FTS_CONTEXT(obj) \
@@ -231,6 +232,10 @@ fts_mailbox_search_init(struct mailbox_transaction_context *t,
 	fctx->result_pool = pool_alloconly_create("fts results", 1024*64);
 	fctx->orig_matches = buffer_create_dynamic(default_pool, 64);
 	fctx->virtual_mailbox = t->box->virtual_vfuncs != NULL;
+	if (fctx->virtual_mailbox) {
+		hash_table_create(&fctx->last_indexed_virtual_uids,
+				  default_pool, 0, str_hash, strcmp);
+	}
 	fctx->enforced = fts_enforced_parse(
 		mail_user_plugin_getenv(t->box->storage->user, "fts_enforced"));
 	i_array_init(&fctx->levels, 8);
@@ -378,14 +383,16 @@ static int fts_mailbox_search_deinit(struct mail_search_context *ctx)
 	int ret = 0;
 
 	if (fctx != NULL) {
+		if (fctx->virtual_mailbox)
+			hash_table_destroy(&fctx->last_indexed_virtual_uids);
 		if (fctx->indexer_ctx != NULL) {
 			if (fts_indexer_deinit(&fctx->indexer_ctx) < 0)
 				ft->failure_reason = "FTS indexing failed";
 		}
 		if (fctx->indexing_timed_out)
 			ret = -1;
-		if (!fctx->fts_lookup_success &&
-		    fctx->enforced != FTS_ENFORCED_NO) {
+		else if (!fctx->fts_lookup_success &&
+			 fctx->enforced != FTS_ENFORCED_NO) {
 			/* FTS lookup failed and we didn't want to fallback to
 			   opening all the mails and searching manually */
 			mail_storage_set_internal_error(ctx->transaction->box->storage);
@@ -427,7 +434,7 @@ static int fts_mail_get_special(struct mail *_mail, enum mail_fetch_field field,
 		i_assert(scores->uid == _mail->uid);
 		(void)i_snprintf(fmail->score, sizeof(fmail->score),
 				 "%f", scores->score);
-			
+
 		*value_r = fmail->score;
 		return 0;
 	}
@@ -557,10 +564,13 @@ static int fts_mail_precache(struct mail *_mail)
 			ft->highest_virtual_uid = _mail->uid;
 	} else if (!ft->indexing) T_BEGIN {
 		/* avoid recursing here from fts_mail_precache_range() */
+		struct event_reason *reason =
+			event_reason_begin("fts:index");
 		ft->indexing = TRUE;
 		ret = fts_mail_index(_mail);
 		i_assert(ft->indexing);
 		ft->indexing = FALSE;
+		event_reason_end(&reason);
 	} T_END;
 	return ret;
 }
@@ -612,6 +622,7 @@ static int fts_transaction_end(struct mailbox_transaction_context *t, const char
 		ret = -1;
 	}
 
+	struct event_reason *reason = event_reason_begin("fts:index");
 	if (ft->precached) {
 		i_assert(flist->update_ctx_refcount > 0);
 		if (--flist->update_ctx_refcount == 0) {
@@ -637,6 +648,7 @@ static int fts_transaction_end(struct mailbox_transaction_context *t, const char
 			       ft->precache_extra_count, t->box->vname);
 		}
 	}
+	event_reason_end(&reason);
 	i_free(ft);
 	return ret;
 }
@@ -791,6 +803,69 @@ static int fts_copy(struct mail_save_context *ctx, struct mail *mail)
 	return 0;
 }
 
+static void fts_mailbox_virtual_match_mail(struct mail_search_context *ctx,
+					   struct mail *mail)
+{
+	struct fts_search_context *fctx = FTS_CONTEXT(ctx);
+	unsigned int idx, be_last_uid;
+
+	if (fctx == NULL || !fctx->fts_lookup_success || !fctx->virtual_mailbox ||
+	    ctx->seq < fctx->first_unindexed_seq)
+		return;
+	/* Table of last indexed UID per backend mailbox */
+	HASH_TABLE_TYPE(virtual_last_indexed) hash_tbl =
+		fctx->last_indexed_virtual_uids;
+
+	struct mail *backend_mail;
+	if (mail->box->mail_vfuncs->get_backend_mail(mail, &backend_mail) < 0)
+		return;
+	const char *box_name = backend_mail->box->vname;
+	/* Get the last indexed UID in the backend mailbox */
+	void *uid_value =
+		hash_table_lookup(fctx->last_indexed_virtual_uids, box_name);
+	if (uid_value == NULL) {
+		/* This backend's last indexed uid is not yet inserted to the table */
+		struct fts_mailbox_list *flist =
+			FTS_LIST_CONTEXT(backend_mail->box->list);
+		if (flist == NULL || flist->failed ||
+		    mailbox_open(backend_mail->box) < 0 ||
+		    fts_backend_get_last_uid(flist->backend, backend_mail->box,
+					     &be_last_uid) < 0) {
+			be_last_uid = 0;
+		} else {
+			const char *vname_copy =
+				p_strdup(fctx->result_pool, backend_mail->box->vname);
+			hash_table_insert(hash_tbl, vname_copy,
+					  POINTER_CAST(be_last_uid + 1));
+		}
+	} else {
+		be_last_uid = POINTER_CAST_TO(uid_value, uint32_t) - 1;
+	}
+	if (backend_mail->uid <= be_last_uid) {
+		/* Mail was already indexed in the backend mailbox.
+		   Apply [non]matches based on the FTS lookup results */
+		struct fts_transaction_context *ft = FTS_CONTEXT_REQUIRE(ctx->transaction);
+
+		if (fctx->next_unindexed_seq == mail->seq) {
+			fctx->next_unindexed_seq++;
+			ft->highest_virtual_uid = mail->uid;
+		}
+		idx = 0;
+		fts_search_apply_results_level(ctx, ctx->args->args, &idx);
+	} else {
+		fctx->virtual_seen_unindexed_gaps = TRUE;
+	}
+}
+
+static int fts_mailbox_search_next_match_mail(struct mail_search_context *ctx,
+					      struct mail *mail)
+{
+	struct fts_mailbox *fbox = FTS_CONTEXT_REQUIRE(ctx->transaction->box);
+
+	fts_mailbox_virtual_match_mail(ctx, mail);
+	return fbox->module_ctx.super.search_next_match_mail(ctx, mail);
+}
+
 void fts_mailbox_allocated(struct mailbox *box)
 {
 	struct fts_mailbox_list *flist = FTS_LIST_CONTEXT(box->list);
@@ -817,6 +892,7 @@ void fts_mailbox_allocated(struct mailbox *box)
 	v->sync_deinit = fts_sync_deinit;
 	v->save_finish = fts_save_finish;
 	v->copy = fts_copy;
+	v->search_next_match_mail = fts_mailbox_search_next_match_mail;
 
 	MODULE_CONTEXT_SET(box, fts_storage_module, fbox);
 }
